//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
#include "wclErrors.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclTimeline"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  edActivationProtocol->Text = "uatest";
  edActivationProtocolDescription->Text = "User Activity Test";
  edActivityId->Text = "TestUserActivity";
  laState->Caption = "<No activity>";
  laProtocol->Caption = "<No activity>";

  edDisplayText->Text = "Hello activities (Test)";
  edDescription->Text = "Test User Activity";

  FActivity = NULL;

  CheckParams();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btRegisterProtocolClick(TObject *Sender)
{
  if (edActivationProtocol->Text == "")
	ShowMessage("Activation protocol can not be an empty string");
  else
  {
	if (edActivationProtocolDescription->Text == "")
	  ShowMessage("Protocol description can not be an empty string");
	else
	{
    #if __CODEGEARC__ <= 600
      int Res = wclTimelineChannel->RegisterProtocol(__classid(TwclTimelineChannel),
        edActivationProtocol->Text, edActivationProtocolDescription->Text);
    #else
	  int Res = wclTimelineChannel->RegisterProtocol(edActivationProtocol->Text,
		edActivationProtocolDescription->Text);
    #endif
	  if (Res != WCL_E_SUCCESS)
		ShowMessage("Failed to register protocol: " + IntToHex(Res, 8));
	  else
		ShowMessage("Protocol registered");
	}
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btUnregisterProtocolClick(TObject *Sender)
{
  if (edActivationProtocol->Text == "")
	ShowMessage("Activation protocol can not be an empty string");
  else
  {
	if (edActivationProtocolDescription->Text == "")
	  ShowMessage("Protocol description can not be an empty string");
	else
	{
    #if __CODEGEARC__ <= 600
	  int Res = wclTimelineChannel->RegisterProtocol(__classid(TwclTimelineChannel),
        edActivationProtocol->Text, edActivationProtocolDescription->Text);
    #else
      int Res = wclTimelineChannel->RegisterProtocol(edActivationProtocol->Text,
		edActivationProtocolDescription->Text);
    #endif
	  if (Res != WCL_E_SUCCESS)
		ShowMessage("Failed to register protocol: " + IntToHex(Res, 8));
	  else
		ShowMessage("Protocol registered");
	}
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btOpenClick(TObject *Sender)
{
  int Res = wclTimelineChannel->Open();
  if (Res != WCL_E_SUCCESS)
	ShowMessage("Open channel failed: 0x" + IntToHex(Res, 8));
  else
	ShowMessage("Channel opened.");
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btCloseClick(TObject *Sender)
{
  DestroyActivity();

  int Res = wclTimelineChannel->Close();
  if (Res != WCL_E_SUCCESS)
	ShowMessage("Close channel failed: 0x" + IntToHex(Res, 8));
  else
	ShowMessage("Channel closed.");
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  DestroyActivity();
  wclTimelineChannel->Close();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btCreateActivityClick(TObject *Sender)
{
  if (FActivity != NULL)
	ShowMessage("Activity has already been created");
  else
  {
	int Res = wclTimelineChannel->CreateActivity(edActivityId->Text, FActivity);
	if (Res != WCL_E_SUCCESS)
	  ShowMessage("Create activity failed: 0x" + IntToHex(Res, 8));
	else
	{
	  ShowMessage("Activity has been created");

	  switch (FActivity->State)
	  {
		case asNew:
		  laState->Caption = "New";
		  laProtocol->Caption = edActivationProtocol->Text;
		  break;

		case asPublished:
		  laState->Caption = "Published";
		  edDescription->Text = FActivity->Description;
		  edDisplayText->Text = FActivity->DisplayText;
		  laProtocol->Caption = FActivity->Protocol;
          edTitleCaption->Text = FActivity->Title;
		  break;
	  }
	}
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btBeginSessionClick(TObject *Sender)
{
  if (FActivity == NULL)
	ShowMessage("Activity has not been created");
  else
  {
	if (FActivity->Active)
	  ShowMessage("Session is active");
	else
	{
	  FActivity->Protocol = edActivationProtocol->Text;
      // Add Activity ID as first parameter so we can open the
      // activity then. You can add additinal params if needed.
	  FActivity->Params = FActivity->Id;
	  FActivity->Description = edDescription->Text;
	  FActivity->DisplayText = edDisplayText->Text;
      FActivity->Title = edTitleCaption->Text;

	  int Res = FActivity->BeginSession();
	  if (Res != WCL_E_SUCCESS)
		ShowMessage("Begin session failed: 0x" + IntToHex(Res, 8));
	  else
		ShowMessage("Session has been started");
	}
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btDestroyActivityClick(TObject *Sender)
{
  if (FActivity == NULL)
	ShowMessage("Activity has not been created");
  else
  {
	DestroyActivity();

	ShowMessage("Activity released");
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btEndSessionClick(TObject *Sender)
{
  if (FActivity == NULL)
	ShowMessage("Activity has not been created");
  else
  {
	int Res = FActivity->EndSession();
	if (Res != WCL_E_SUCCESS)
	  ShowMessage("End session failed: 0x" + IntToHex(Res, 8));
	else
	  ShowMessage("Session has been stopped");
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::DestroyActivity()
{
  if (FActivity != NULL)
  {
	FActivity->Free();
	FActivity = NULL;

	laState->Caption = "<No activity>";
	laProtocol->Caption = "<No activity>";
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::CheckParams()
{
  // Check application params.
  if (ParamCount() > 0)
  {
	String Str = ParamStr(1);
	// Make sure it is correct params.
	int p = Str.Pos(":");
	if (p > 0)
	{
	  // Extract ActivityID.
	  Str = Str.SubString(p + 1, Str.Length() - p);
	  // Start activity with given ID.
	  if (Str != "")
	  {
		int Res = wclTimelineChannel->Open();
		if (Res == WCL_E_SUCCESS)
		{
		  Res = wclTimelineChannel->CreateActivity(Str, FActivity);
		  if (Res == WCL_E_SUCCESS)
		  {
			ShowMessage("Application started from Activity. Activity opened.");
			laState->Caption = "Published";
			edActivityId->Text = Str;
			edDescription->Text = FActivity->Description;
			edDisplayText->Text = FActivity->DisplayText;
			laProtocol->Caption = FActivity->Protocol;
			edTitleCaption->Text = FActivity->Title;
		  }
		  else
			wclTimelineChannel->Close();
		}
	  }
	}
  }
}
//---------------------------------------------------------------------------
